`ifndef EX_TYPE_PKG
  `ifndef EX_TYPE_DEFINED
    `define EX_TYPE_DEFINED
    `include "ex_type.svh"
  `endif
`endif

module control_test_m ;

  timeunit        1ns ;
  timeprecision 100ps ;

`ifdef USE_IMPORT
  import ex_type_pkg::* ;
`endif

  localparam time PERIOD = 20 ;

  logic load_ac   ; // control load accumulator
  logic mem_rd    ; // control memory read
  logic mem_wr    ; // control memory write
  logic inc_pc    ; // control increment program counter
  logic load_pc   ; // control load program counter
  logic load_ir   ; // control load instruction register
  logic halt      ; // control CPU halt
  opcode_e opcode ; // control_test operation (enumerated)
  logic zero      ; // control_test accumulator value is 0
  logic rst_      ; // control_test reset (asynch low)
  logic clk       ; // control_test clock
  logic clk2      ; // control_test clock/2
  logic fetch     ; // control_test clock/4

  int unsigned stimulus_num=0 ; // stimulus index
  int unsigned response_num=0 ; // response index
  logic [3:0]  stimulus_reg=0 ; // stimulus vector (current)
  logic [6:0]  response_net   ; // response vector (current)
  logic [3:0]  stimulus_mem[1:64]  ; // stimulus vectors ($readmemb)
  logic [6:0]  response_mem[1:512] ; // response vectors ($readmemb)

  // Generate Clock
  initial begin clk=0; clk2=1; fetch=1; end
  always #(PERIOD/2) clk = ~clk ;
  always @(negedge clk) clk2 <= ~clk2 ;
  always @(negedge clk) if ( clk2==0 ) fetch <= ~fetch ;

  // Instantiate Controller
  control_m            control  ( .* ) ;

  assign response_net = { mem_rd,load_ir,halt,inc_pc,load_ac,load_pc,mem_wr } ;
  assign opcode = opcode_e'(stimulus_reg[2:0]) ;
  assign zero   = stimulus_reg[3] ;

  // Monitor Results
  initial
    begin
      $timeformat ( -9, 0, "ns", 6 ) ;
      $monitor ( "%t rst_=%b ph=%d zer=%b op=%s rd=%b l_ir=%b hlt=%b inc=%b l_ac=%b l_pc=%b wr=%b",
      $time, rst_, control.phase, zero, opcode.name(),
      mem_rd, load_ir, halt, inc_pc, load_ac, load_pc, mem_wr ) ;
      #(600 * PERIOD)
      $display ( "CONTROLLER TEST TIMEOUT" ) ;
      $finish ;
    end

  // Verify Results
  always @ ( clk )
    #(PERIOD/4)
    if ( stimulus_num != 0 )
      if (response_net !== response_mem[++response_num])
        begin
          logic [6:0] temp ;
          temp = response_mem[response_num] ;
          $display ( "CONTROLLER TEST FAILED" ) ;
          $display ( "results should be . . . . . . . .  rd=%b l_ir=%b hlt=%b inc=%b l_ac=%b l_pc=%b wr=%b\n",
          temp[6], temp[5], temp[4], temp[3], temp[2], temp[1], temp[0] ) ;
          $finish ;
        end

  // Apply Stimulus
  initial
    begin
      $readmemb ( "stimulus.pat", stimulus_mem ) ;
      $readmemb ( "response.pat", response_mem ) ;
      // delay reset -- note both clock edges are active!
      @(negedge clk) #(PERIOD/4) rst_ <= 1 ;
      @(negedge clk) #(PERIOD/4) rst_ <= 0 ;
      @(negedge clk) #(PERIOD/4) rst_ <= 1 ;
      do
        // coincident with INST_FETCH phase
        @(posedge clk iff (clk2 && fetch))
          stimulus_reg <= stimulus_mem[++stimulus_num] ;
      while ( stimulus_num <= 64 ) ;
      $display ( "CONTROLLER TEST PASSED" ) ;
      $finish ;
    end

endmodule : control_test_m
